==> formatter_suite-8.x-1.0.tar.gz: formatter_suite/src/Plugin/Field/FieldFormatter/GeneralEmailFormatter.php <==
<?php

namespace Drupal\formatter_suite\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Render\FormattableMarkup;

use Drupal\formatter_suite\Branding;

/**
 * Formats an email address.
 *
 * The email address may be shown as:
 *   - The plain text address.
 *   - The email address and a mailto link to the address.
 *   - Custom text and a mailto link to the address.
 *
 * @ingroup formatter_suite
 *
 * @FieldFormatter(
 *   id          = "formatter_suite_general_email",
 *   label       = @Translation("Formatter Suite - General Email address"),
 *   weight      = 1000,
 *   field_types = {
 *     "email",
 *   }
 * )
 */
class GeneralEmailFormatter extends FormatterBase {

  /*---------------------------------------------------------------------
   *
   * Configuration.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns an array of formatting styles.
   *
   * @return string[]
   *   Returns an associative array with internal names as keys and
   *   human-readable translated names as values.
   */
  protected static function getEmailStyles() {
    return [
      'plain'        => t('Email address as text'),
      'mailto'       => t('Email address with mail-to link'),
      'custommailto' => t('Custom text with mail-to link'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'emailStyle' => 'mailto',
      'linkText'   => t('Email'),
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $this->sanitizeSettings();

    $summary = [];
    switch ($this->getSetting('emailStyle')) {
      default:
      case 'plain':
        $summary[] = $this->t('Email address');
        break;

      case 'mailto':
        $summary[] = $this->t('Email address link');
        break;

      case 'custommailto':
        $summary[] = $this->t('Text with Email address link');
        break;
    }

    return $summary;
  }

  /*---------------------------------------------------------------------
   *
   * Settings form.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns a brief description of the formatter.
   *
   * @return string
   *   Returns a brief translated description of the formatter.
   */
  protected function getDescription() {
    return t("Formats an email address as plain text, or as a \"mailto\" link that starts the user's email application.");
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $formState) {
    // Get the parent's form.
    $elements = parent::settingsForm($form, $formState);

    // Add branding.
    $elements = Branding::addFieldFormatterBranding($elements);
    $elements['#attached']['library'][] =
      'formatter_suite/formatter_suite.fieldformatter';

    // Below, some checkboxes and select choices show/hide other form
    // elements. We use Drupal's obscure 'states' feature, which adds
    // Javascript to elements to auto show/hide based upon a set of
    // simple conditions.
    //
    // Those conditions need to reference the form elements to check
    // (e.g. a checkbox), but the element ID and name are automatically
    // generated by the parent form. We cannot set them, or predict them,
    // so we cannot use them. We could use a class, but this form may be
    // shown multiple times on the same page, so a simple class would not be
    // unique. Instead, we create classes for this form only by adding a
    // random number marker to the end of the class name.
    $marker = rand();

    // Add description.
    //
    // Use a large negative weight to insure it comes first.
    $elements['description'] = [
      '#type'          => 'html_tag',
      '#tag'           => 'div',
      '#value'         => $this->getDescription(),
      '#weight'        => -1000,
      '#attributes'    => [
        'class'        => [
          'formatter_suite-settings-description',
        ],
      ],
    ];

    $weight = 0;

    // Prompt for each setting.
    $elements['emailStyle'] = [
      '#title'         => $this->t('Address style'),
      '#type'          => 'select',
      '#options'       => $this->getEmailStyles(),
      '#default_value' => $this->getSetting('emailStyle'),
      '#weight'        => $weight++,
      '#wrapper_attributes' => [
        'class'        => [
          'formatter_suite-general-email-style',
        ],
      ],
      '#attributes'    => [
        'class'        => [
          'emailStyle-' . $marker,
        ],
      ],
    ];

    $elements['linkText'] = [
      '#title'         => $this->t('Link text'),
      '#type'          => 'textfield',
      '#default_value' => $this->getSetting('linkText'),
      '#weight'        => $weight++,
      '#wrapper_attributes' => [
        'class'        => [
          'formatter_suite-general-email-link-text',
        ],
      ],
      '#states'        => [
        'visible'    => [
          '.emailStyle-' . $marker => [
            'value'    => 'custommailto',
          ],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * Sanitize settings to insure that they are safe and valid.
   *
   * @internal
   * Drupal's class hierarchy for plugins and their settings does not
   * include a 'validate' function, like that for other classes with forms.
   * Validation must therefore occur on use, rather than on form submission.
   * @endinternal
   */
  protected function sanitizeSettings() {
    // Get settings.
    $emailStyle = $this->getSetting('emailStyle');
    $linkText   = $this->getSetting('linkText');
    $defaults   = $this->defaultSettings();

    // Sanitize & validate.
    $emailStyles = $this->getEmailStyles();
    if (empty($emailStyle) === TRUE ||
        isset($emailStyles[$emailStyle]) === FALSE) {
      $emailStyle = $defaults['emailStyle'];
      $this->setSetting('emailStyle', $emailStyle);
    }

    if (empty($linkText) === TRUE) {
      // Link text is further sanitized during use.
      $linkText = $defaults['linkText'];
      $this->setSetting('linkText', $linkText);
    }
  }

  /*---------------------------------------------------------------------
   *
   * View.
   *
   *---------------------------------------------------------------------*/

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Security: The link text is entered by an administrator.
    // It may legitimately include HTML entities and minor HTML, but
    // it should not include dangerous HTML.
    //
    // Because it may include HTML, we cannot pass it directly to t()
    // or let a TWIG template use {{ }}, both of which will process
    // the text and corrupt any entered HTML or HTML entities.
    //
    // We therefore use an Xss filter to remove any egreggious HTML
    // (such as scripts), and then FormattableMarkup() to mark the
    // resulting text as safe.
    $linkText = Xss::filterAdmin($this->getSetting('linkText'));
    $linkText = new FormattableMarkup($linkText, []);

    // Security: The email address is entered by a user. It should be
    // an address of the form NAME@DOMAIN and should not contain any
    // HTML or HTML entities.
    //
    // Passing the email address to the 'link' render type, or using
    // 'plain_text', sends it to a TWIG template which uses {{ }} to
    // show the value. This automatically sanitizes the value to remove
    // or escape any HTML.  There is nothing further we need to do here.
    $elements = [];

    switch ($this->getSetting('emailStyle')) {
      case 'plain':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#plain_text' => $item->value,
          ];
        }
        break;

      case 'mailto':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#type'  => 'link',
            '#title' => $item->value,
            '#url'   => Url::fromUri('mailto:' . $item->value),
          ];
        }
        break;

      case 'custommailto':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#type'  => 'link',
            '#title' => $linkText,
            '#url'   => Url::fromUri('mailto:' . $item->value),
          ];
        }
        break;
    }

    return $elements;
  }

}

==> formatter_suite-8.x-1.1.tar.gz: formatter_suite/src/Plugin/Field/FieldFormatter/GeneralEmailFormatter.php <==
<?php

namespace Drupal\formatter_suite\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Render\FormattableMarkup;

use Drupal\formatter_suite\Branding;

/**
 * Formats an email address.
 *
 * The email address may be shown as:
 *   - The plain text address.
 *   - The email address and a mailto link to the address.
 *   - Custom text and a mailto link to the address.
 *
 * @ingroup formatter_suite
 *
 * @FieldFormatter(
 *   id          = "formatter_suite_general_email",
 *   label       = @Translation("Formatter Suite - General Email address"),
 *   weight      = 1000,
 *   field_types = {
 *     "email",
 *   }
 * )
 */
class GeneralEmailFormatter extends FormatterBase {

  /*---------------------------------------------------------------------
   *
   * Configuration.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns an array of formatting styles.
   *
   * @return string[]
   *   Returns an associative array with internal names as keys and
   *   human-readable translated names as values.
   */
  protected static function getEmailStyles() {
    return [
      'plain'        => t('Email address as text'),
      'mailto'       => t('Email address with mail-to link'),
      'custommailto' => t('Custom text with mail-to link'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array_merge(
      [
        'emailStyle' => 'mailto',
        'linkText'   => t('Email'),
      ],
      parent::defaultSettings());
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $this->sanitizeSettings();

    $summary = [];
    switch ($this->getSetting('emailStyle')) {
      default:
      case 'plain':
        $summary[] = $this->t('Email address');
        break;

      case 'mailto':
        $summary[] = $this->t('Email address link');
        break;

      case 'custommailto':
        $summary[] = $this->t('Text with Email address link');
        break;
    }

    return $summary;
  }

  /*---------------------------------------------------------------------
   *
   * Settings form.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns a brief description of the formatter.
   *
   * @return string
   *   Returns a brief translated description of the formatter.
   */
  protected function getDescription() {
    return $this->t("Formats an email address as plain text, or as a \"mailto\" link that starts the user's email application.");
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $formState) {
    // Get the parent's form.
    $elements = parent::settingsForm($form, $formState);

    // Add branding.
    $elements = Branding::addFieldFormatterBranding($elements);
    $elements['#attached']['library'][] =
      'formatter_suite/formatter_suite.fieldformatter';

    // Below, some checkboxes and select choices show/hide other form
    // elements. We use Drupal's obscure 'states' feature, which adds
    // Javascript to elements to auto show/hide based upon a set of
    // simple conditions.
    //
    // Those conditions need to reference the form elements to check
    // (e.g. a checkbox), but the element ID and name are automatically
    // generated by the parent form. We cannot set them, or predict them,
    // so we cannot use them. We could use a class, but this form may be
    // shown multiple times on the same page, so a simple class would not be
    // unique. Instead, we create classes for this form only by adding a
    // random number marker to the end of the class name.
    $marker = rand();

    // Add description.
    //
    // Use a large negative weight to insure it comes first.
    $elements['description'] = [
      '#type'          => 'html_tag',
      '#tag'           => 'div',
      '#value'         => $this->getDescription(),
      '#weight'        => -1000,
      '#attributes'    => [
        'class'        => [
          'formatter_suite-settings-description',
        ],
      ],
    ];

    $weight = 0;

    // Prompt for each setting.
    $elements['emailStyle'] = [
      '#title'         => $this->t('Address style'),
      '#type'          => 'select',
      '#options'       => $this->getEmailStyles(),
      '#default_value' => $this->getSetting('emailStyle'),
      '#weight'        => $weight++,
      '#wrapper_attributes' => [
        'class'        => [
          'formatter_suite-general-email-style',
        ],
      ],
      '#attributes'    => [
        'class'        => [
          'emailStyle-' . $marker,
        ],
      ],
    ];

    $elements['linkText'] = [
      '#title'         => $this->t('Link text'),
      '#type'          => 'textfield',
      '#default_value' => $this->getSetting('linkText'),
      '#weight'        => $weight++,
      '#wrapper_attributes' => [
        'class'        => [
          'formatter_suite-general-email-link-text',
        ],
      ],
      '#states'        => [
        'visible'    => [
          '.emailStyle-' . $marker => [
            'value'    => 'custommailto',
          ],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * Sanitize settings to insure that they are safe and valid.
   *
   * @internal
   * Drupal's class hierarchy for plugins and their settings does not
   * include a 'validate' function, like that for other classes with forms.
   * Validation must therefore occur on use, rather than on form submission.
   * @endinternal
   */
  protected function sanitizeSettings() {
    // Get settings.
    $emailStyle = $this->getSetting('emailStyle');
    $linkText   = $this->getSetting('linkText');
    $defaults   = $this->defaultSettings();

    // Sanitize & validate.
    $emailStyles = $this->getEmailStyles();
    if (empty($emailStyle) === TRUE ||
        isset($emailStyles[$emailStyle]) === FALSE) {
      $emailStyle = $defaults['emailStyle'];
      $this->setSetting('emailStyle', $emailStyle);
    }

    if (empty($linkText) === TRUE) {
      // Link text is further sanitized during use.
      $linkText = $defaults['linkText'];
      $this->setSetting('linkText', $linkText);
    }
  }

  /*---------------------------------------------------------------------
   *
   * View.
   *
   *---------------------------------------------------------------------*/

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Security: The link text is entered by an administrator.
    // It may legitimately include HTML entities and minor HTML, but
    // it should not include dangerous HTML.
    //
    // Because it may include HTML, we cannot pass it directly to t()
    // or let a TWIG template use {{ }}, both of which will process
    // the text and corrupt any entered HTML or HTML entities.
    //
    // We therefore use an Xss filter to remove any egreggious HTML
    // (such as scripts), and then FormattableMarkup() to mark the
    // resulting text as safe.
    $linkText = Xss::filterAdmin($this->getSetting('linkText'));
    $linkText = new FormattableMarkup($linkText, []);

    // Security: The email address is entered by a user. It should be
    // an address of the form NAME@DOMAIN and should not contain any
    // HTML or HTML entities.
    //
    // Passing the email address to the 'link' render type, or using
    // 'plain_text', sends it to a TWIG template which uses {{ }} to
    // show the value. This automatically sanitizes the value to remove
    // or escape any HTML.  There is nothing further we need to do here.
    $elements = [];

    switch ($this->getSetting('emailStyle')) {
      case 'plain':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#plain_text' => $item->value,
          ];
        }
        break;

      case 'mailto':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#type'  => 'link',
            '#title' => $item->value,
            '#url'   => Url::fromUri('mailto:' . $item->value),
          ];
        }
        break;

      case 'custommailto':
        foreach ($items as $delta => $item) {
          $elements[$delta] = [
            '#type'  => 'link',
            '#title' => $linkText,
            '#url'   => Url::fromUri('mailto:' . $item->value),
          ];
        }
        break;
    }

    return $elements;
  }

}
